﻿//
// Modbus UDP C# sample for IR API Library.
// This demo application is a Visual Studi 2019 C# WPF project for IR-712-MTCP.
// 
// Release Note:
//   [v2.2.0.0], 2022-June-02
//



using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Threading;
using IRAPI_Ns;

namespace IrAPI_CSharpWPFDemo
{
    /// <summary>
    /// Interaction logic for IRMUDPDemoWin0.xaml
    /// </summary>
    public partial class IRMUDPDemoWin0 : Window
    {
        public IRMUDPDemoWin0()
        {
            InitializeComponent();
        }

        private int socketHandle = 0;           // Socket handle for Modbus UDP protocol.
        private Byte mbNetID = 1;               // Modbus Net ID
        private UInt32 serverIP = 0;            // IP address of the IR-712-MTCP
        private Byte moduleCnt = 0;             // Quantity of searched module
        private Byte[] IROutputChArr = new Byte[3] { 0x01, 0x02, 0x03 };    // Combinations of IR output channels.
        private String[] deviceNameArr = new string[11];                    // Strings array for IR device names.
        private String[] cmdNameArr = new String[512];                      // Strings array for IR cmd names.


        private int[] irCmdQtyInDevice = new int[3] { 1, 2, 3 };
        private int IrDeviceTotalQty = 3,
                    IrCmdCurrTotalCmd = 6;

        private int[] IrCmdSummaryArr = new int[13];

        private Thread thDwLDIR;    // Thread for downloading IR cmds to the IR module and updating IR cmd numbers in the Textbox "txtIRCmdQtyCurr".
        private Thread thLDIR;      // Thread for loading IR cmds from the IR module and updating IR cmd numbers in the Textbox "txtIRCmdQtyCurr".



        private void IRMUDPUtilWin0_Loaded(object sender, RoutedEventArgs e)
        {
            this.Height = 600;

            // Init combobox of the address type
            cboAddrType.Items.Clear();
            cboAddrType.Items.Add("Static");
            cboAddrType.Items.Add("DHCP");
            cboAddrType.SelectedIndex = 0;

            cboAddrType_S.Items.Clear();
            cboAddrType_S.Items.Add("Static");
            cboAddrType_S.Items.Add("DHCP");


            // Init the combobox of the Modbus Net ID
            cboMBNetID1.Items.Clear();
            cboMBNetID.Items.Clear();
            cboMBNetID_S.Items.Clear();
            for (int i = 1; i < 248; i++)
            {
                cboMBNetID1.Items.Add(i.ToString());
                cboMBNetID.Items.Add(i.ToString());
                cboMBNetID_S.Items.Add(i.ToString());
            }

            cboMBNetID1.SelectedIndex = 0;
            cboMBNetID.SelectedIndex = 0;


            // Init combobox of HW Mode
            cboHwMode_S.Items.Clear();
            cboHwMode_S.Items.Add("OP");
            cboHwMode_S.Items.Add("Init/FW");



            // Init. Write action combobox.
            cboWriteAction.Items.Add("No Set");
            cboWriteAction.Items.Add("Set & Reboot");
            cboWriteAction.Items.Add("Set & No Reboot");
            cboWriteAction.SelectedIndex = 1;

            // Init. the combobox of IR command number
            cboIRCmdNo.Items.Clear();
            cboIRCmdNo2.Items.Clear();
            for (int i = 1; i < 513; i++)
            {
                cboIRCmdNo.Items.Add(i.ToString());
                cboIRCmdNo2.Items.Add(i.ToString());
            }
            cboIRCmdNo.SelectedIndex = 0;
            cboIRCmdNo2.SelectedIndex = 0;

            // Init. the combobox of IR output channel
            cboIROutputCh.Items.Clear();
            cboIROutputCh.Items.Add("1");
            cboIROutputCh.Items.Add("2");
            cboIROutputCh.Items.Add("1 + 2");
            cboIROutputCh.SelectedIndex = 0;

            cboIROutputCh2.Items.Clear();
            cboIROutputCh2.Items.Add("1");
            cboIROutputCh2.Items.Add("2");
            cboIROutputCh2.Items.Add("1 + 2");
            cboIROutputCh2.SelectedIndex = 0;


            // Init. Device name comboboxe
            deviceNameArr[0] = "TV";                          // IR device #1
            deviceNameArr[1] = "Audio";                       // IR device #2
            deviceNameArr[2] = "A/C";           //"空調";     // IR device #3

            cboDeviceName.Items.Clear();
            for (int i = 0; i < deviceNameArr.Length; i++)
            {
                cboDeviceName.Items.Add((i + 1).ToString() + "_" + deviceNameArr[i]);
            }
            cboDeviceName.SelectedIndex = 0;


            // Init. IR command name combobox
            cmdNameArr[0] = "Power Toggle";     //"開關";     // IR cmd #1
            cmdNameArr[1] = "Vol+";                           // IR cmd #2
            cmdNameArr[2] = "Vol-";                           // IR cmd #3
            cmdNameArr[3] = "Power Toggle";     //"開關";     // IR cmd #4
            cmdNameArr[4] = "Temperature+";     //"溫度+";    // IR cmd #5
            cmdNameArr[5] = "Wind Direction";   //"風向";     // IR cmd #6

            cboIrCmdNameArr.Items.Clear();
            for (int i = 0; i < cmdNameArr.Length; i++)
            {
                cboIrCmdNameArr.Items.Add((i + 1).ToString() + "_" + cmdNameArr[i]);
            }
            cboIrCmdNameArr.SelectedIndex = 0;


            // Init. the IR command summary data in the IR Cmd Data buffer in the IR API library.
            int rtn = 0;
            ushort moduleIndex = 0x21;          // Module index. 0x21: IR-712-MTCP
            ushort[] dataArr = new ushort[13];  // Array for IR command summary.
            IrCmdSummaryArr[0] = dataArr[0] = 3;                     // IR device quantity (#1, #2, #3)
            IrCmdSummaryArr[1] = dataArr[1] = 6;                     // Total IR command quantity
            IrCmdSummaryArr[2] = dataArr[2] = 1;                     // IR command quantity for IR device #1
            IrCmdSummaryArr[3] = dataArr[3] = 2;                     // IR command quantity for IR device #2
            IrCmdSummaryArr[4] = dataArr[4] = 3;                     // IR command quantity for IR device #3

            rtn = IR_API_Class.IR_SetIRCmdSummaryBuffer(moduleIndex, dataArr, (ushort)dataArr.Length);  // Set IR cmd summary to buffer.



            // Init. the IR device name & command name data in the IR Cmd Data buffer in the IR API library.
            Byte[] IrDevNameArr = new Byte[100];
            Byte[] IrCmdNameArr = new Byte[100];

            for (int i = 0; i < dataArr[0]; i++)
            {
                switch (i)
                {
                    case 0: // for IR device #1
                        for (int j = 0; j < dataArr[2]; j++)
                        {
                            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(deviceNameArr[i]);
                            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(cmdNameArr[j]);
                            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                                    (UInt16)(j + 1),
                                    IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                                    IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                                );
                        }
                        break;
                    case 1: // for IR device #2
                        for (int j = 0; j < dataArr[3]; j++)
                        {
                            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(deviceNameArr[i]);
                            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(cmdNameArr[j + dataArr[2]]);
                            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                                    (UInt16)(j + dataArr[2] + 1),
                                    IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                                    IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                                );
                        }
                        break;
                    case 2: // for IR device #3
                        for (int j = 0; j < dataArr[4]; j++)
                        {
                            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(deviceNameArr[i]);
                            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(cmdNameArr[j + dataArr[2] + dataArr[3]]);
                            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                                    (UInt16)(j + dataArr[2] + dataArr[3] + 1),
                                    IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                                    IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                                );
                        }
                        break;
                    default:
                        break;
                }

            }


            // Init. Module Info Index combobox
            cboModuleInfoIndex.Items.Clear();
            for (int i = 0; i < 10; i++)
            {
                cboModuleInfoIndex.Items.Add(i.ToString());
            }
            cboModuleInfoIndex.SelectedIndex = 0;



            // Init rectangle
            Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.White);


        }



        private void btnGetInterfaces_Click(object sender, RoutedEventArgs e)
        {
            int iNICCount = 0;
            IR_API_Class.IR_ChkNIC(ref iNICCount);
            MessageBox.Show("Number of NICs = " + iNICCount.ToString(), "Check NIC");

            // Show NIC description
            byte[] description = new byte[133];
            int actualLen = 0;
            UInt32 IPAddr = 0;
            Byte NB0, NB1, NB2, NB3;  // Network address:= NB0.NB1.NB2.NB3
            String showStr;

            for (int i = 0; i < iNICCount; i++)
            {
                IR_API_Class.IR_GetNICInfo((byte)i, description, description.Length, ref actualLen, ref IPAddr);
                NB0 = (Byte)(IPAddr >> 24);
                NB1 = (Byte)(IPAddr >> 16);
                NB2 = (Byte)(IPAddr >> 8);
                NB3 = (Byte)IPAddr;
                showStr = string.Format("NIC IPAddr = 0x" + IPAddr.ToString("X") + " := "
                        + NB0.ToString() + "." + NB1.ToString() + "." + NB2.ToString() + "." + NB3.ToString()
                        + "\n\rNIC Description = {0}"
                        + "\n\r\n\rIs this IP address of NIC for IR module connection?",
                        Encoding.ASCII.GetString(description, 0, actualLen));

                MessageBoxResult mResult = MessageBox.Show(showStr, "Check IP Address of Network Adapter", MessageBoxButton.YesNo, MessageBoxImage.Information);
                if (mResult == MessageBoxResult.Yes)
                {
                    txtIPNICByte0.Text = NB0.ToString();
                    txtIPNICByte1.Text = NB1.ToString();
                    txtIPNICByte2.Text = NB2.ToString();
                    txtIPNICByte3.Text = NB3.ToString();
                    break;
                }
            }
        }



        private void btnSearchModule_Click(object sender, RoutedEventArgs e)
        {
            moduleCnt = 0;
            UInt32 NicIPAddr = 0xC0A80F7C;         // ETH NIC = 192.168.15.124
            NicIPAddr = UInt32.Parse(txtIPNICByte0.Text) << 24
                    | UInt32.Parse(txtIPNICByte1.Text) << 16
                    | UInt32.Parse(txtIPNICByte2.Text) << 8
                    | UInt32.Parse(txtIPNICByte3.Text);
            IR_API_Class.IR_EthSearchModules(NicIPAddr, ref moduleCnt, 500);

            MessageBox.Show("Number of IR modules = " + moduleCnt.ToString());
        }



        private void btnGetModuleInfo_Click(object sender, RoutedEventArgs e)
        {
            if (moduleCnt == 0)
            {
                MessageBox.Show("No modules found!\nClick the \"Search IR-712-MTCP\" first.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            UInt16 mInfoIndex = UInt16.Parse(cboModuleInfoIndex.Text);
            IR_API_Class.SearchRespEth moduleInfoStruct = new IR_API_Class.SearchRespEth();

            // Get the module info by the mInfoIndex which indexes the found IR modules.
            IR_API_Class.IR_EthGetSearchModuleInfo(
                mInfoIndex,
                ref moduleInfoStruct
                );


            // Module Name
            txtModuleName_S.Text = moduleInfoStruct.ucModIndex == 0x21 ? "IR-712-MTCP" : "???";

            // Address Type
            switch (moduleInfoStruct.ucAddrType)
            {
                case 0:
                    cboAddrType_S.SelectedIndex = 0;
                    break;
                case 1:
                    cboAddrType_S.SelectedIndex = 1;
                    break;
                default:
                    cboAddrType_S.SelectedIndex = -1;
                    break;
            }

            // IP address
            txtIPByte0_S.Text = ((Byte)(moduleInfoStruct.ulIPAddr >> 24)).ToString();
            txtIPByte1_S.Text = ((Byte)(moduleInfoStruct.ulIPAddr >> 16)).ToString();
            txtIPByte2_S.Text = ((Byte)(moduleInfoStruct.ulIPAddr >> 8)).ToString();
            txtIPByte3_S.Text = ((Byte)moduleInfoStruct.ulIPAddr).ToString();
            txtIPConnByte0.Text = ((Byte)(moduleInfoStruct.ulIPAddr >> 24)).ToString();
            txtIPConnByte1.Text = ((Byte)(moduleInfoStruct.ulIPAddr >> 16)).ToString();
            txtIPConnByte2.Text = ((Byte)(moduleInfoStruct.ulIPAddr >> 8)).ToString();
            txtIPConnByte3.Text = ((Byte)moduleInfoStruct.ulIPAddr).ToString();

            // Subnet Mask
            txtMaskByte0_S.Text = ((Byte)(moduleInfoStruct.ulMaskAddr >> 24)).ToString();
            txtMaskByte1_S.Text = ((Byte)(moduleInfoStruct.ulMaskAddr >> 16)).ToString();
            txtMaskByte2_S.Text = ((Byte)(moduleInfoStruct.ulMaskAddr >> 8)).ToString();
            txtMaskByte3_S.Text = ((Byte)moduleInfoStruct.ulMaskAddr).ToString();

            // Gateway address
            txtGWByte0_S.Text = ((Byte)(moduleInfoStruct.ulGWAddr >> 24)).ToString();
            txtGWByte1_S.Text = ((Byte)(moduleInfoStruct.ulGWAddr >> 16)).ToString();
            txtGWByte2_S.Text = ((Byte)(moduleInfoStruct.ulGWAddr >> 8)).ToString();
            txtGWByte3_S.Text = ((Byte)moduleInfoStruct.ulGWAddr).ToString();

            // Modbus Net ID
            cboMBNetID_S.SelectedIndex = moduleInfoStruct.ucNetID - 1;
            cboMBNetID1.SelectedIndex = moduleInfoStruct.ucNetID - 1;

            // Ethernet idle timeout (ms)
            txtIdleTOut_S.Text = moduleInfoStruct.ulIdleTOut.ToString();

            // Firmware Version
            txtFwVer_S.Text = ((Byte)(moduleInfoStruct.ulFwVer >> 16)).ToString() + "."
                            + ((Byte)(moduleInfoStruct.ulFwVer >> 8)).ToString() + "."
                            + ((Byte)(moduleInfoStruct.ulFwVer)).ToString();
            // HW Mode
            cboHwMode_S.SelectedIndex = (moduleInfoStruct.ucHwMode == 0) ? 0 : 1;

            // MAC Address
            txtMACByte0_S.Text = ((Byte)(moduleInfoStruct.dwlMacAddr >> 40)).ToString("x2");
            txtMACByte1_S.Text = ((Byte)(moduleInfoStruct.dwlMacAddr >> 32)).ToString("x2");
            txtMACByte2_S.Text = ((Byte)(moduleInfoStruct.dwlMacAddr >> 24)).ToString("x2");
            txtMACByte3_S.Text = ((Byte)(moduleInfoStruct.dwlMacAddr >> 16)).ToString("x2");
            txtMACByte4_S.Text = ((Byte)(moduleInfoStruct.dwlMacAddr >> 8)).ToString("x2");
            txtMACByte5_S.Text = ((Byte)moduleInfoStruct.dwlMacAddr).ToString("x2");

        }



        private void btnClearInfoCtrls_Click(object sender, RoutedEventArgs e)
        {
            cboAddrType_S.SelectedIndex = -1;
            txtModuleName_S.Text = String.Empty;
            txtIPByte0_S.Text = String.Empty;
            txtIPByte1_S.Text = String.Empty;
            txtIPByte2_S.Text = String.Empty;
            txtIPByte3_S.Text = String.Empty;
            txtMaskByte0_S.Text = String.Empty;
            txtMaskByte1_S.Text = String.Empty;
            txtMaskByte2_S.Text = String.Empty;
            txtMaskByte3_S.Text = String.Empty;
            txtGWByte0_S.Text = String.Empty;
            txtGWByte1_S.Text = String.Empty;
            txtGWByte2_S.Text = String.Empty;
            txtGWByte3_S.Text = String.Empty;
            cboMBNetID_S.SelectedIndex = 0;
            txtIdleTOut_S.Text = String.Empty;
            cboHwMode_S.SelectedIndex = -1;
            txtMACByte0_S.Text = String.Empty;
            txtMACByte1_S.Text = String.Empty;
            txtMACByte2_S.Text = String.Empty;
            txtMACByte3_S.Text = String.Empty;
            txtMACByte4_S.Text = String.Empty;
            txtMACByte5_S.Text = String.Empty;
        }

        

        private void btnUDPNew_Click(object sender, RoutedEventArgs e)
        {   // New a socket handle with communicaton by default route table.
            serverIP = UInt32.Parse(txtIPConnByte0.Text) << 24
                    | UInt32.Parse(txtIPConnByte1.Text) << 16
                    | UInt32.Parse(txtIPConnByte2.Text) << 8
                    | UInt32.Parse(txtIPConnByte3.Text);

            socketHandle = IR_API_Class.IR_MUDP_UDPNew(serverIP, Byte.Parse(cboMBNetID1.Text));
            MessageBox.Show("UDP Socket handle = " + socketHandle.ToString());
        }



        private void btnUDPNew1_Click(object sender, RoutedEventArgs e)
        {   // New a socket handle with communicaton by selected network interface (NIC IP).

            UInt32 NicIPAddr = 0xC0A80F7B;         // ETH NIC = 192.168.15.123
            NicIPAddr = UInt32.Parse(txtIPNICByte0.Text) << 24
                    | UInt32.Parse(txtIPNICByte1.Text) << 16
                    | UInt32.Parse(txtIPNICByte2.Text) << 8
                    | UInt32.Parse(txtIPNICByte3.Text);

            serverIP = UInt32.Parse(txtIPConnByte0.Text) << 24
                    | UInt32.Parse(txtIPConnByte1.Text) << 16
                    | UInt32.Parse(txtIPConnByte2.Text) << 8
                    | UInt32.Parse(txtIPConnByte3.Text);

            socketHandle = IR_API_Class.IR_MUDP_UDPNew1(NicIPAddr, serverIP, Byte.Parse(cboMBNetID1.Text));
            MessageBox.Show("UDP Socket handle = " + socketHandle.ToString());
        }



        private void btnCloseUDPSocket_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_MUDP_UDPClose(socketHandle);
            MessageBox.Show("Close UDP Socket handle rtn = " + rtn.ToString());
        }



        private void btnChkModuleName_Click(object sender, RoutedEventArgs e)
        {
            UInt16 moduleIndex = 0;

            int rtn = IR_API_Class.IR_MUDP_ChkModuleName(socketHandle, ref moduleIndex);

            if (moduleIndex == 0x21)
            {
                txtModuleIndex.Text = "IR-712-MTCP";
            }

            //txtModuleIndex.Text = "0x" + moduleIndex.ToString("X2");
        }



        private void btnMUDPGetBasicSettings_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            IR_API_Class.BasicSetParamsMTCP mtcpBasicSetStruct = new IR_API_Class.BasicSetParamsMTCP();

            mbNetID = Byte.Parse(cboMBNetID1.Text);

            rtn = IR_API_Class.IR_MUDP_GetBasicSetting(
                socketHandle,
                ref mtcpBasicSetStruct
                );

            if (rtn == IR_API_Class.IR_E_OK)
            {

                // Address Type
                switch (mtcpBasicSetStruct.ucAddrType)
                {
                    case 0:
                        cboAddrType.SelectedIndex = 0;
                        break;
                    case 1:
                        cboAddrType.SelectedIndex = 1;
                        break;
                    default:
                        cboAddrType.SelectedIndex = -1;
                        break;
                }

                // IP address
                txtIPByte0.Text = ((Byte)(mtcpBasicSetStruct.ulIPAddr >> 24)).ToString();
                txtIPByte1.Text = ((Byte)(mtcpBasicSetStruct.ulIPAddr >> 16)).ToString();
                txtIPByte2.Text = ((Byte)(mtcpBasicSetStruct.ulIPAddr >> 8)).ToString();
                txtIPByte3.Text = ((Byte)mtcpBasicSetStruct.ulIPAddr).ToString();

                // Subnet Mask
                txtMaskByte0.Text = ((Byte)(mtcpBasicSetStruct.ulMaskAddr >> 24)).ToString();
                txtMaskByte1.Text = ((Byte)(mtcpBasicSetStruct.ulMaskAddr >> 16)).ToString();
                txtMaskByte2.Text = ((Byte)(mtcpBasicSetStruct.ulMaskAddr >> 8)).ToString();
                txtMaskByte3.Text = ((Byte)mtcpBasicSetStruct.ulMaskAddr).ToString();

                // Gateway address
                txtGWByte0.Text = ((Byte)(mtcpBasicSetStruct.ulGWAddr >> 24)).ToString();
                txtGWByte1.Text = ((Byte)(mtcpBasicSetStruct.ulGWAddr >> 16)).ToString();
                txtGWByte2.Text = ((Byte)(mtcpBasicSetStruct.ulGWAddr >> 8)).ToString();
                txtGWByte3.Text = ((Byte)mtcpBasicSetStruct.ulGWAddr).ToString();

                // Modbus Net ID
                cboMBNetID.SelectedIndex = mtcpBasicSetStruct.ucNetID - 1;

                // Ethernet idle timeout (ms)
                txtIdleTOut.Text = mtcpBasicSetStruct.ulIdleTOut.ToString();

                // Gaptime (ms)
                txtGapTime.Text = mtcpBasicSetStruct.usGapTime.ToString();

                MessageBox.Show("Get Basic Settings OK", "OK");
            }
            else
            {
                MessageBox.Show("Error in IR_MUDP_GetBasicSetting()", "Fail");
            }
        }



        private void btnMUDPSetBasicSettings_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            UInt32 AddressU32 = 0;   // host byte order

            // Check Set Option
            if (cboWriteAction.SelectedIndex == -1)
            {
                MessageBox.Show("Please select \"Set Options\" combobox first!", "Warning", MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }



            IR_API_Class.BasicSetParamsMTCP mtcpBasicSetStruct = new IR_API_Class.BasicSetParamsMTCP();

            // Address Type: Static(0) or DHCP(1)
            mtcpBasicSetStruct.ucAddrType = (Byte)cboAddrType.SelectedIndex;

            // IP address
            AddressU32 =
                       UInt32.Parse(txtIPByte3.Text)
                       | (UInt32.Parse(txtIPByte2.Text) << 8)
                       | (UInt32.Parse(txtIPByte1.Text) << 16)
                       | (UInt32.Parse(txtIPByte0.Text) << 24);

            mtcpBasicSetStruct.ulIPAddr = AddressU32;

            // Subnet Mask
            AddressU32 = UInt32.Parse(txtMaskByte3.Text)
                       | (UInt32.Parse(txtMaskByte2.Text) << 8)
                       | (UInt32.Parse(txtMaskByte1.Text) << 16)
                       | (UInt32.Parse(txtMaskByte0.Text) << 24);
            mtcpBasicSetStruct.ulMaskAddr = AddressU32;

            // Gateway Mask
            AddressU32 = UInt32.Parse(txtGWByte3.Text)
                       | (UInt32.Parse(txtGWByte2.Text) << 8)
                       | (UInt32.Parse(txtGWByte1.Text) << 16)
                       | (UInt32.Parse(txtGWByte0.Text) << 24);
            mtcpBasicSetStruct.ulGWAddr = AddressU32;


            // Modbus Net ID
            mtcpBasicSetStruct.ucNetID = (Byte)(cboMBNetID.SelectedIndex + 1);

            // Eth Idle Timeout
            mtcpBasicSetStruct.ulIdleTOut = UInt32.Parse(txtIdleTOut.Text);

            // Gaptime
            mtcpBasicSetStruct.usGapTime = UInt16.Parse(txtGapTime.Text);

            rtn = IR_API_Class.IR_MUDP_SetBasicSetting(socketHandle, mtcpBasicSetStruct, (Byte)cboWriteAction.SelectedIndex);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Set error", "Error");
            }
            else
            {
                MessageBox.Show("Set OK", "OK");
                switch (cboWriteAction.SelectedIndex)
                {
                    case 1: // Set and reboot
                        txtIPConnByte0.Text = txtIPByte0.Text;
                        txtIPConnByte1.Text = txtIPByte1.Text;
                        txtIPConnByte2.Text = txtIPByte2.Text;
                        txtIPConnByte3.Text = txtIPByte3.Text;
                        cboMBNetID1.SelectedIndex = cboMBNetID.SelectedIndex;
                        break;
                }
            }

        }



        private void btnClearSetting_Click(object sender, RoutedEventArgs e)
        {
            cboAddrType.SelectedIndex = 0;
            txtIPByte0.Text = String.Empty;
            txtIPByte1.Text = String.Empty;
            txtIPByte2.Text = String.Empty;
            txtIPByte3.Text = String.Empty;
            txtMaskByte0.Text = String.Empty;
            txtMaskByte1.Text = String.Empty;
            txtMaskByte2.Text = String.Empty;
            txtMaskByte3.Text = String.Empty;
            txtGWByte0.Text = String.Empty;
            txtGWByte1.Text = String.Empty;
            txtGWByte2.Text = String.Empty;
            txtGWByte3.Text = String.Empty;
            cboMBNetID.SelectedIndex = 0;
            txtIdleTOut.Text = String.Empty;
            txtGapTime.Text = String.Empty;
        }



        private void txtBasicSettingDefault_Click(object sender, RoutedEventArgs e)
        {
            cboAddrType.SelectedIndex = 0;
            txtIPByte0.Text = "192";
            txtIPByte1.Text = "168";
            txtIPByte2.Text = "255";
            txtIPByte3.Text = "1";
            txtMaskByte0.Text = "255";
            txtMaskByte1.Text = "255";
            txtMaskByte2.Text = "0";
            txtMaskByte3.Text = "0";
            txtGWByte0.Text = "192";
            txtGWByte1.Text = "168";
            txtGWByte2.Text = "0";
            txtGWByte3.Text = "254";
            cboMBNetID.SelectedIndex = 0;
            txtIdleTOut.Text = "0";
            txtGapTime.Text = "72";
        }



        private void btnReboot_Click(object sender, RoutedEventArgs e)
        {
            // Reboot the IR module.
            int rtn = IR_API_Class.IR_MUDP_RebootIRModule(socketHandle);
            MessageBox.Show("Reboot Module return = " + rtn.ToString());
        }



        private void btnGetIRCmdSummaryBuff_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            ushort moduleIndex = 0x21;    // 0x21: IR-712-MTCP
            ushort[] dataArr = new ushort[13];
            string dataStr = String.Empty;

            rtn = IR_API_Class.IR_GetIRCmdSummaryBuffer(moduleIndex, dataArr, (ushort)dataArr.Length);

            // Show the IR command summary.
            for (int i = 0; i < dataArr.Length; i++)
            {
                if (i == (dataArr.Length - 1))
                {
                    dataStr += (dataArr[i].ToString());
                    //dataStr += (dataArr[i].ToString("X4"));
                }
                else
                {
                    dataStr += (dataArr[i].ToString() + "-");
                    //dataStr += (dataArr[i].ToString("X4") + "-");
                }

                IrCmdSummaryArr[i] = dataArr[i];
            }

            txtDeviceQty_G.Text = dataArr[0].ToString();
            IrDeviceTotalQty = dataArr[0];
            txtIRCmdQtyTotal_G.Text = dataArr[1].ToString();
            IrCmdCurrTotalCmd = dataArr[1];
            txtIRCmdQtyDevice1_G.Text = dataArr[2].ToString();
            txtIRCmdQtyDevice2_G.Text = dataArr[3].ToString();
            txtIRCmdQtyDevice3_G.Text = dataArr[4].ToString();
            txtIRCmdQtyDevice4_G.Text = dataArr[5].ToString();
            txtIRCmdQtyDevice5_G.Text = dataArr[6].ToString();
            txtIRCmdQtyDevice6_G.Text = dataArr[7].ToString();
            txtIRCmdQtyDevice7_G.Text = dataArr[8].ToString();
            txtIRCmdQtyDevice8_G.Text = dataArr[9].ToString();
            txtIRCmdQtyDevice9_G.Text = dataArr[10].ToString();
            txtIRCmdQtyDevice10_G.Text = dataArr[11].ToString();
            txtIRCmdQtyDevice11_G.Text = dataArr[12].ToString();

            MessageBox.Show(dataStr, "Get dataArr");
        }



        private void btnSetIRCmdSummaryBuff_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            ushort moduleIndex = 0x21;    // 0x21: IR-712-MTCP
            ushort[] dataArr = new ushort[13];
            string dataStr = String.Empty;

            dataArr[0] = UInt16.Parse(txtDeviceQty.Text);
            IrDeviceTotalQty = dataArr[0];
            dataArr[1] = UInt16.Parse(txtIRCmdQtyTotal.Text);
            IrCmdCurrTotalCmd = dataArr[1];
            dataArr[2] = UInt16.Parse(txtIRCmdQtyDevice1.Text);
            dataArr[3] = UInt16.Parse(txtIRCmdQtyDevice2.Text);
            dataArr[4] = UInt16.Parse(txtIRCmdQtyDevice3.Text);
            dataArr[5] = UInt16.Parse(txtIRCmdQtyDevice4.Text);
            dataArr[6] = UInt16.Parse(txtIRCmdQtyDevice5.Text);
            dataArr[7] = UInt16.Parse(txtIRCmdQtyDevice6.Text);
            dataArr[8] = UInt16.Parse(txtIRCmdQtyDevice7.Text);
            dataArr[9] = UInt16.Parse(txtIRCmdQtyDevice8.Text);
            dataArr[10] = UInt16.Parse(txtIRCmdQtyDevice9.Text);
            dataArr[11] = UInt16.Parse(txtIRCmdQtyDevice10.Text);
            dataArr[12] = UInt16.Parse(txtIRCmdQtyDevice11.Text);


            rtn = IR_API_Class.IR_SetIRCmdSummaryBuffer(moduleIndex, dataArr, (ushort)dataArr.Length);

            for (int i = 0; i < dataArr.Length; i++)
            {
                dataStr += (dataArr[i].ToString("X4") + " ");

                IrCmdSummaryArr[i] = dataArr[i];
            }

            MessageBox.Show(dataStr, "Set IR Cmd Summary to the library buffer.");
        }



        private void btnGetIrDevCmdNameInBuffer_Click(object sender, RoutedEventArgs e)
        {
            // Note: The IR learning data buffer of the API library was initilized 
            //       with 3 IR device names and 6 IR command names by IR_SetIrDevCmdNameInBuffer()  
            //       in the IRMUDPUtilWin0_Loaded() event handler.

            int rtn = 0;
            Byte[] IrDevNameArr = new Byte[100];
            Byte[] IrCmdNameArr = new Byte[100];
            UInt16 IrDevNameLength = 0;
            UInt16 IrCmdNameLength = 0;
            rtn = IR_API_Class.IR_GetIrDevCmdNameInBuffer((UInt16)(cboIRCmdNo2.SelectedIndex + 1), IrDevNameArr, (UInt16)IrDevNameArr.Length, ref IrDevNameLength,
                   IrCmdNameArr, (UInt16)IrCmdNameArr.Length, ref IrCmdNameLength);

            txtIRDevNameGet.Text = Encoding.UTF8.GetString(IrDevNameArr, 0, IrDevNameLength);
            txtIRCmdNameGet.Text = Encoding.UTF8.GetString(IrCmdNameArr, 0, IrCmdNameLength);

        }


        private void btnSetIrDevCmdNameInBuffer_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            Byte[] IrDevNameArrTemp = Encoding.UTF8.GetBytes(txtIRDevNameSet.Text);
            Byte[] IrCmdNameArrTemp = Encoding.UTF8.GetBytes(txtIRCmdNameSet.Text);

            rtn = IR_API_Class.IR_SetIrDevCmdNameInBuffer(
                (UInt16)(cboIRCmdNo2.SelectedIndex + 1),
                IrDevNameArrTemp, (UInt16)IrDevNameArrTemp.Length,
                IrCmdNameArrTemp, (UInt16)IrCmdNameArrTemp.Length
                );
        }



        private void cboDeviceName_DropDownClosed(object sender, EventArgs e)
        {
            ComboBox cbo = sender as ComboBox;
            int index = 0;

            for (int i = 0; i < cbo.SelectedIndex; i++)
            { index += IrCmdSummaryArr[i + 2]; }

            cboIrCmdNameArr.SelectedIndex = index;   // Show the first IR cmd name for the IR device #.
        }



        private void cboIrCmdNameArr_DropDownClosed(object sender, EventArgs e)
        {
            int rtn = 0;
            rtn = IR_API_Class.IR_ChkIrDataExistInBuffer((UInt16)(cboIrCmdNameArr.SelectedIndex + 1));


            if (rtn == IR_API_Class.IR_E_IRCMD_DATA_EXIST)
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.Red);
            }
            else
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.White);
            }
        }



        private void cboIrCmdNameArr_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            int rtn = 0;
            rtn = IR_API_Class.IR_ChkIrDataExistInBuffer((UInt16)(cboIrCmdNameArr.SelectedIndex + 1));


            if (rtn == IR_API_Class.IR_E_IRCMD_DATA_EXIST)
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.Red);
            }
            else
            {
                Rect1.Fill = new SolidColorBrush(System.Windows.Media.Colors.White);
            }
        }



        private void btnRefreshIRDevCmdNameInLibBuf_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            Byte[] IrDevNameArr = new Byte[100];
            Byte[] IrCmdNameArr = new Byte[100];
            UInt16 IrDevNameLength = 0;
            UInt16 IrCmdNameLength = 0;
            UInt16 irCmdNo = 0;

            btnGetIRCmdSummaryBuff_Click(sender, e);    // Get IR cmd summary from the IR library buffer.

            cboDeviceName.Items.Clear();
            cboIrCmdNameArr.Items.Clear();

            // Initialize the items of IR device names & IR cmd name (comments) comboboxes.  
            for (int i = 0; i < IrCmdSummaryArr[0]; i++)
            {
                for (int j = 0; j < IrCmdSummaryArr[i + 2]; j++)
                {

                    rtn = IR_API_Class.IR_GetIrDevCmdNameInBuffer(++irCmdNo, IrDevNameArr, (UInt16)IrDevNameArr.Length, ref IrDevNameLength,
                           IrCmdNameArr, (UInt16)IrCmdNameArr.Length, ref IrCmdNameLength);

                    cboIrCmdNameArr.Items.Add(irCmdNo.ToString() + "_" + Encoding.UTF8.GetString(IrCmdNameArr, 0, IrCmdNameLength));
                }

                cboDeviceName.Items.Add((i + 1) + "_" + Encoding.UTF8.GetString(IrDevNameArr, 0, IrDevNameLength));
            }

            if ((cboDeviceName.Items.Count != 0) && (cboIrCmdNameArr.Items.Count != 0))
            {
                cboDeviceName.SelectedIndex = cboIrCmdNameArr.SelectedIndex = 0;
            }
        }



        private void btnMUdpLrnON_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_MUDP_LrnModeSet(socketHandle, 1);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("IR-Learn-ON failed", "Error");
            }

        }



        private void btnMUdpLrnOFF_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_MUDP_LrnModeSet(socketHandle, 0);


            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("IR-Learn-OFF failed", "Error");
            }
        }



        private void btnIsLearnOK_Click(object sender, RoutedEventArgs e)
        {
            UInt16 isLearnOK = 0;   // Variable to indicate if learning an IR cmd is completed. 0=>not yet. 1=>completed.
            int rtn = IR_API_Class.IR_MUDP_IsIRLearnOK(socketHandle, ref isLearnOK, int.Parse(txtRecvTout.Text));
            txtIsLrnOKRtn.Text = rtn.ToString();
            MessageBox.Show("Is IR-Learning OK? => " + ((isLearnOK == 1) ? "OK" : "Not OK"), "Is IR learning OK?");
        }



        private void btnGetIRCmdAfterLrn_Click(object sender, RoutedEventArgs e)
        {
            int cboCurrSelectedIndex = cboIrCmdNameArr.SelectedIndex;
            int rtn = IR_API_Class.IR_MUDP_GetIRCmdAfterLearn(socketHandle, (UInt16)(cboCurrSelectedIndex + 1));

            if (rtn == IR_API_Class.IR_E_OK)
            {
                cboIrCmdNameArr_DropDownClosed(sender, e);
                MessageBox.Show("Call IR_MUDP_GetIRCmdAfterLearn() OK.", "OK");
            }
            else
            {
                MessageBox.Show("Error in calling IR_MUDP_GetIRCmdAfterLearn()", "Error");
            }
        }



        private void btnRunCmd_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_MUDP_RunCommand(socketHandle, (UInt16)(cboIrCmdNameArr.SelectedIndex + 1), IROutputChArr[cboIROutputCh.SelectedIndex]);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Error in calling IR_MUDP_GetIRCmdAfterLearn()", "Error");
            }
        }



        private void btnClearOneIRDataInLib_Click(object sender, RoutedEventArgs e)
        {
            UInt16 moduleIndex = 0x21;   // IR-712-MTCP
            int cboIndex = cboIrCmdNameArr.SelectedIndex;
            int rtn = IR_API_Class.IR_ClearOneIRLrnDataInAPI((UInt16)(cboIndex + 1), moduleIndex);

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("IR_ClearOneIRLrnDataInAPI() return error code = " + rtn.ToString(), "IR_ClearOneIRLrnDataInAPI Error");
            }

            cboIrCmdNameArr_DropDownClosed(sender, e);
        }



        private void btnSetIRCmdSummary_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            UInt16 moduleIndex = 0x21;  // 0x21: ModuleIndex of IR-712-MTCP
            UInt16[] dataArr = new UInt16[13];

            MessageBoxResult msgRst = MessageBox.Show(
                "Before pressing this button, press the \"btnSetIRCmdSummaryBuff\" button in advance in group Step 4 to update the IR cmd summary if necessary. \nThis button will write the IR command summary in the library buffer \nto the flash memory of the IR module.\nAre you sure to proceed?\n",
                "Warning", MessageBoxButton.YesNo,
                MessageBoxImage.Warning);

            if (msgRst == MessageBoxResult.No)
            {
                return;
            }


            btnSetIRCmdSummaryBuff_Click(sender, e);

            rtn = IR_API_Class.IR_MUDP_SetIRCmdSummary(socketHandle, moduleIndex);

            //dataArr[0] = Byte.Parse(txtDeviceQty.Text);
            //dataArr[1] = Byte.Parse(txtIRCmdQtyTotal.Text);
            //dataArr[2] = Byte.Parse(txtIRCmdQtyDevice1.Text);
            //dataArr[3] = Byte.Parse(txtIRCmdQtyDevice2.Text);
            //dataArr[4] = Byte.Parse(txtIRCmdQtyDevice3.Text);
            //dataArr[5] = Byte.Parse(txtIRCmdQtyDevice4.Text);
            //dataArr[6] = Byte.Parse(txtIRCmdQtyDevice5.Text);
            //dataArr[7] = Byte.Parse(txtIRCmdQtyDevice6.Text);
            //dataArr[8] = Byte.Parse(txtIRCmdQtyDevice7.Text);
            //dataArr[9] = Byte.Parse(txtIRCmdQtyDevice8.Text);
            //dataArr[10] = Byte.Parse(txtIRCmdQtyDevice9.Text);
            //dataArr[11] = Byte.Parse(txtIRCmdQtyDevice10.Text);
            //dataArr[12] = Byte.Parse(txtIRCmdQtyDevice11.Text);

            //rtn = IR_API_Class.IR_MUDP_SetIRCmdSummary1(socketHandle, moduleIndex, dataArr, (UInt16)dataArr.Length);



            txtSetIRCmdSummaryRtn.Text = rtn.ToString();

            //MessageBox.Show("a[0]=" + dataArr[0].ToString("X2")
            //              + "\na[1]=" + dataArr[1].ToString("X2")
            //              + "\na[2]=" + dataArr[2].ToString("X2")
            //              + "\na[3]=" + dataArr[3].ToString("X2")
            //              + "\na[4]=" + dataArr[4].ToString("X2")
            //              + "\na[5]=" + dataArr[5].ToString("X2")
            //              + "\na[6]=" + dataArr[6].ToString("X2")
            //              + "\na[7]=" + dataArr[7].ToString("X2")
            //              );
        }



        private void btnDLOneIRDataToModule_Click(object sender, RoutedEventArgs e)
        {
            thDwLDIR = new Thread(new ThreadStart(DownloadIRProcess));
            thDwLDIR.Start();

        }


        void DownloadIRProcess()
        {
            int rtn = 0;
            for (int i = 0; i < IrCmdCurrTotalCmd; i++)
            {
                rtn = IR_API_Class.IR_MUDP_DLOneIRDataToModule(socketHandle, (UInt16)(i + 1));

                if (rtn != IR_API_Class.IR_E_OK)
                {
                    MessageBox.Show("Failed to download IR cmds", "Download Error");
                    return;
                }

                Dispatcher.BeginInvoke(
                    (Action)(() =>
                    {
                        txtIRCmdQtyCurr.Text = (i + 1).ToString();
                    }
                    ));
                Thread.Sleep(50);
            }

            MessageBox.Show("Download " + IrCmdCurrTotalCmd.ToString() + " IR cmds to the IR modules successfully.", "Download OK");
        }



        private void btnEmitIRCmd_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_MUDP_EmitIrSignal(socketHandle, (UInt16)(cboIRCmdNo.SelectedIndex + 1), IROutputChArr[cboIROutputCh2.SelectedIndex]);
        }



        private void btnGetIRCmdSummary_Click(object sender, RoutedEventArgs e)
        {
            int rtn = 0;
            UInt16 moduleIndex = 0x21;  // 0x21: ModuleIndex of IR-712-MTCP
            UInt16[] dataArr = new UInt16[13];
            UInt16[] dataArr2 = new UInt16[13];
            UInt16[] dataArr3 = new UInt16[13];
            String dataStr = String.Empty;

            rtn = IR_API_Class.IR_MUDP_GetIRCmdSummary(socketHandle, moduleIndex, dataArr, (UInt16)dataArr.Length);

            // Show the IR command summary.
            for (int i = 0; i < dataArr.Length; i++)
            {
                if (i == (dataArr.Length - 1))
                {
                    dataStr += (dataArr[i].ToString());
                    //dataStr += (dataArr[i].ToString("X4"));
                }
                else
                {
                    dataStr += (dataArr[i].ToString() + "-");
                    //dataStr += (dataArr[i].ToString("X4") + "-");
                }

                IrCmdSummaryArr[i] = dataArr[i];
            }

            txtGetIRCmdSummaryRtn.Text = rtn.ToString();
            txtGetIRCmdSummary.Text = dataStr;

            IrDeviceTotalQty = dataArr[0];
            IrCmdCurrTotalCmd = dataArr[1];

        }



        private void btnLDOneIRDataToAPI_Click(object sender, RoutedEventArgs e)
        {
            thLDIR = new Thread(new ThreadStart(LoadIRProcess));
            thLDIR.Start();
        }


        void LoadIRProcess()
        {
            int rtn = 0;
            for (int i = 0; i < IrCmdCurrTotalCmd; i++)
            {
                rtn = IR_API_Class.IR_MUDP_LDOneIRDataToAPI(socketHandle, (UInt16)(i + 1));

                if (rtn != IR_API_Class.IR_E_OK)
                {
                    MessageBox.Show("Failed to load IR cmds", "load Error");
                    return;
                }

                Dispatcher.BeginInvoke(
                    (Action)(() =>
                    {
                        txtIRCmdQtyCurr.Text = (i + 1).ToString();
                    }
                    ));
                Thread.Sleep(50);
            }

            MessageBox.Show("Load " + IrCmdCurrTotalCmd.ToString() + " IR cmds back to the IR API library successfully.", "Load back OK");
        }



        private void btnSaveIRCmdToFile_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_SaveIRCmdsToFile(txtFilePath_Save.Text);
            txtSaveIRCmdToFileRtn.Text = rtn.ToString();

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Save failed.");
            }
        }



        private void btnLoadIRCmdToFile_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_LoadIRCmdsFromFile(txtFilePath_Load.Text);
            txtLoadIRCmdToFileRtn.Text = rtn.ToString();

            if (rtn != IR_API_Class.IR_E_OK)
            {
                MessageBox.Show("Load failed.");
            }
        }



        private void txtIRCmdQtyDevice1_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox txt = sender as TextBox;
            int irCmdOtyTotal = 0;
            int irCmdQtyDevice = 0;
            int irDeviceQtyTotal = 0;


            // #1 device - IR cmd qty
            if (txtIRCmdQtyDevice1 == null || int.TryParse(txtIRCmdQtyDevice1.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);


            // #2 device - IR cmd qty
            if (txtIRCmdQtyDevice2 == null || int.TryParse(txtIRCmdQtyDevice2.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #3 device - IR cmd qty
            if (txtIRCmdQtyDevice3 == null || int.TryParse(txtIRCmdQtyDevice3.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #4 device - IR cmd qty
            if (txtIRCmdQtyDevice4 == null || int.TryParse(txtIRCmdQtyDevice4.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #5 device - IR cmd qty
            if (txtIRCmdQtyDevice5 == null || int.TryParse(txtIRCmdQtyDevice5.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #6 device - IR cmd qty
            if (txtIRCmdQtyDevice6 == null || int.TryParse(txtIRCmdQtyDevice6.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #7 device - IR cmd qty
            if (txtIRCmdQtyDevice7 == null || int.TryParse(txtIRCmdQtyDevice7.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #8 device - IR cmd qty
            if (txtIRCmdQtyDevice8 == null || int.TryParse(txtIRCmdQtyDevice8.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #9 device - IR cmd qty
            if (txtIRCmdQtyDevice9 == null || int.TryParse(txtIRCmdQtyDevice9.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #10 device - IR cmd qty
            if (txtIRCmdQtyDevice10 == null || int.TryParse(txtIRCmdQtyDevice10.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            // #11 device - IR cmd qty
            if (txtIRCmdQtyDevice11 == null || int.TryParse(txtIRCmdQtyDevice11.Text, out irCmdQtyDevice) == false)
            {
                irCmdQtyDevice = 0;
            }
            irCmdOtyTotal += irCmdQtyDevice;
            irDeviceQtyTotal += (irCmdQtyDevice > 0 ? 1 : 0);

            txtIRCmdQtyTotal.Text = irCmdOtyTotal.ToString();
            txtDeviceQty.Text = irDeviceQtyTotal.ToString();
        }



        private void btnGetMBRecvTOut_Click(object sender, RoutedEventArgs e)
        {
            int rtn = IR_API_Class.IR_MUDP_GetModbusRecvTimeout();
            txtRecvTout1.Text = rtn.ToString();
        }



        private void btnSetMBRecvTOut_Click(object sender, RoutedEventArgs e)
        {
            IR_API_Class.IR_MUDP_SetModbusRecvTimeout((UInt16)int.Parse(txtRecvTout1.Text));
        }
        
    }
}
