﻿
/* ******************************************************************************************
 * C# Wrapper of the IR API Library
 * 
 * Namespace: IRAPI_Ns
 * 
 * Class name: IR_API_Class 
 * 
 * Description:
 *   C# wrapper class of the IR API library for IR learning remote module (IR-210/IR-712A/IR-712-MTCP).
 *
 * Release Note:
 *   [v2.2.0.0], 2022-June-02
 *   
 * 
 * ****************************************************************************************** */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace IRAPI_Ns
{
	class IR_API_Class
	{
		///
		/// Return Code
		///
		public const Int32 IR_E_OK =                              0x00;   /* No errors. The function runs successfully.                                                                                      */
		public const Int32 IR_E_COM_STATUS_AVAIL =                0x01;   /* The COM port is available and not open.                                                                                         */
		public const Int32 IR_E_IRCMD_DATA_EMPTY =                0x00;   /* NO IR data exist for the specified IR cmd number in the API buffer.                                                             */
		public const Int32 IR_E_IRCMD_DATA_EXIST =                0x01;   /* IR data exist for the specified IR cmd number in the API buffer.                                                                */
		public const Int32 IR_E_RESET_OK =                        0x02;   /* Reset IR module sucessfully.                                                                                                    */

		public const Int32 IR_E_FUNC_ERR =                        0x11;   /* Function error.                                                                                                                 */
		public const Int32 IR_E_COM_STATUS_ERR =                  0x12;   /* The COM port is not available or is already open.                                                                               */
		public const Int32 IR_E_OPEN_COM_ERR =                    0x13;   /* Open COM port error                                                                                                             */
		public const Int32 IR_E_ILL_BAUD_IDX =                    0x14;   /* Illegal baud rate index                                                                                                         */
		public const Int32 IR_E_ILL_DATA_BIT =                    0x15;   /* Illegal data bits                                                                                                               */
		public const Int32 IR_E_ILL_STOP_BIT =                    0x16;   /* Illegal stop bits                                                                                                               */
		public const Int32 IR_E_ILL_PARITY =                      0x17;   /* Illegal parity                                                                                                                  */
		public const Int32 IR_E_MODNAME_INVALID =                 0x18;   /* Get invalid module name.                                                                                                        */
		public const Int32 IR_E_SEND_MBCMD_ERR =                  0x19;   /* Error in sending/receiving Modbus command.                                                                                      */
		public const Int32 IR_E_SEND_MBCMD_FLASH_ERR =            0x1A;   /* Error in sending/receiving Modbus command to write data to Flash.                                                               */
		public const Int32 IR_E_ILL_SUMMARY_LEN =                 0x1B;   /* Illegal length of the array for IR command summary (should be 13)                                                               */
		public const Int32 IR_E_DEVICE_QTY_ERR =                  0x1C;   /* The device quantity exceeds the maximum.                                                                                        */
		public const Int32 IR_E_IRCMD_TOTAL_QTY_ERR =             0x1D;   /* The element "Total quantity of IR commands" in the IR cmd summary exceeds the maximum.                                          */
		public const Int32 IR_E_IRCMD_SUM_QTY_ERR =               0x1E;   /* The sum of all device's IR command quantity exceeds the maximum.                                                                */
		public const Int32 IR_E_ILL_MODULE_INDEX =                0x1F;   /* Illegal module index                                                                                                            */
		public const Int32 IR_E_ILL_IRCMD_NO =                    0x20;   /* Illegal IR command number                                                                                                       */
		public const Int32 IR_E_DEVICE_NAME_ARR_UNDERSIZE =       0x21;   /* The length of the device name array is too small as an argument to a function.                                                  */
		public const Int32 IR_E_IRCMD_NAME_ARR_UNDERSIZE =        0x22;   /* The length of the IR cmd name array is too small as an argument to a function.                                                  */
		public const Int32 IR_E_DEVICE_NAME_ARR_OVERSIZE =        0x23;   /* The length of the device name is too large as an argument to a function.                                                        */
		public const Int32 IR_E_IRCMD_NAME_ARR_OVERSIZE =         0x24;   /* The length of the IR cmd name is too large as an argument to a function.                                                        */
		public const Int32 IR_E_LRNCMD_LEN_EXCEED_MAX =           0x25;   /* The length of the IR cmd learning data exceeds the maximum.                                                                     */
		public const Int32 IR_E_RESET_COM_ERR =                   0x26;   /* COM error in sending reset message to the IR module.                                                                            */
		public const Int32 IR_E_RESET_MODULE_FAIL =               0x27;   /* Resetting the IR module is not successful.                                                                                      */
		public const Int32 IR_E_IRCMD_TOTALQTY_NOMATCH =          0x28;   /* The totl quantity of IR cmds do not match the sum of IR cmds in all devices in the array of IR cmd summary                      */
		public const Int32 IR_E_OPEN_FILE_ERROR =                 0x29;   /* Open file error																		                                           */
		public const Int32 IR_E_CLOSE_FILE_ERROR =                0x2A;   /* Close file error																		                                           */
		public const Int32 IR_E_IR_FILE_FORMAT_ERR =              0x2B;   /* Format error in IR learning data file												                                           */
		public const Int32 IR_E_SOCKET_INVALID =                  (-1);   /* SOCKET Error																			                                           */
		public const Int32 IR_E_SOCKET_BIND_ERR =                 0x2C;   /* Error in binding socket with the network address of NIC                                                                         */
		public const Int32 IR_E_SOCKET_CONN_TOUT =                0x2D;   /* Connection timeout by Modbus TCP                                                                                                */
		public const Int32 IR_E_SOCKET_RECV_TOUT =                0x2E;   /* Socket receive timeout                                                                                                          */



		// Structure definition for the parameter of function IR_Get/SetBasicSetting()
		// This structure colllects basic setting parameters of IR modules with Modbus RTU comminucation.
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
		public struct BasicSetParams
		{
			public Byte ucBaudrateIndex;  // Baud rate Index. [6]:9600 bps, [10]:115200 bps
			public Byte ucData;           // Data bits  always 8
			public Byte ucParity;         // Parity  0=>None  1=>Odd  2=>Even
			public Byte ucStop;           // Stop bits.  1 => 1 stopbit,  2 => 2 stopbits
			public Byte ucNetID;          // Modbus Net ID of IR modules ( 1 ~ 247 )
			public UInt16 usMBRespDlyT;   // Modbus response delay time (ms), default = 3 ms
			public UInt16 usGapTime;      // Gap time (ms), default= 72 ms, range = 72 ~ 200 ms

		};


		// Structure definition for the parameter of function IR_MTCP_Get/SetBasicSetting() & IR_MUDP_Get/SetBasicSetting().
		// This structure colllects basic setting parameters of IR modules with Modbus TCP/UDP communication.
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
		public struct BasicSetParamsMTCP
		{
			public UInt32 ulIPAddr;       // IP Addres.   192.168.0.1 = > 0xC0A80001
			public UInt32 ulMaskAddr;	  // Mask Addres.  255.255.0.0 = > 0xFFFF0000
			public UInt32 ulGWAddr;       // Gateway address. 192.168.0.254 = > 0xC0A800FE
			public UInt32 ulIdleTOut;     // Idle communication timeout (mimnute)
			public Byte ucNetID;          // Modbus Net ID of IR modules ( 1 ~ 247 )
			public Byte ucAddrType;       // Address type. 0: static  1: DHCP
			public UInt16 usGapTime;      // Gap time (ms), default= 72 ms, range = 72 ~ 200 ms
		};



		// Structure definition for the parameter of function IR_EthGetSearchModuleInfo()
		// This structure colllects parameters responsed from IR modules with Ethernet interface.
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
		public struct SearchRespEth
		{
			public UInt64 dwlMacAddr;    // MAC address 1a:2b:3c:4d:5e:6f => 0x00001A2B3C4D5E6F
			public UInt32 ulIPAddr;      // IP Addres.   192.168.0.1 => 0xC0A80001
			public UInt32 ulMaskAddr;	  // Mask Addres.  255.255.0.0 => 0xFFFF0000
			public UInt32 ulGWAddr;      // Gateway address. 192.168.0.254 => 0xC0A800FE 
			public UInt32 ulFwVer;       // Firmware version of the module. v1.2.3 => 0x00010203 
			public UInt32 ulIdleTOut;    // Idle communication timeout (unit: mimnute, 0 ~ 65535)
			public Byte ucNetID;         // Modbus Net ID of IR modules ( 1 ~ 247 ) 
			public Byte ucAddrType;      // Address type. 0: static  1: DHCP
			public Byte ucHwMode;	      // Mode: OP(=0) or FW/Init (=1) mode
			public Byte ucModIndex;      // Module Index.   0x21 => "IR-712-MTCP"	
		};




		// @1
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern UInt32 IR_GetAPIVersion();



		// @2
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern UInt32 IR_GetAPIDate();



		// @3
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetComPortStatus(
			Byte ucPort
			);



		// @4
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_OpenCom(
			Byte comPortNo,
			Byte ucBaudIndex,
			Byte ucData,
			Byte ucParity,
			Byte ucStop
			);



		// @5
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_CloseCom(
			Byte ucPort
			);



		// @6
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_SetBasicSetting(
			Byte ucPort,
			Byte ucNetID,
			[In]BasicSetParams SBasicSetting
			);



		// @7
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetBasicSetting(
			Byte ucPort,
			Byte ucNetID,
			ref BasicSetParams SBasicSetting
			);






		// @8
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_ChkModuleName(
			Byte ucPort,
			Byte ucNetID,
			ref UInt16 pwModuleIndex
			);


		// @9
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_LrnModeSet(
			Byte ucPort,
			Byte ucNetID,
			Byte ucMode
			);


		// @10
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_IsIRLearnOK(
			Byte ucPort,
			ref UInt16 pusIsOK,
			UInt16 usTimeout
			);


		// @11
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_SetIRCmdSummary(
			Byte ucPort,
			Byte usNetID,
			UInt16 usIrModuleIndex
			);



		// @30
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_SetIRCmdSummary1(
			Byte ucPort,
			Byte usNetID,
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] UInt16[] dataArr,
			UInt16 usInElementCnt
			);


		// @12
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetIRCmdSummary(
			Byte ucPort,
			Byte usNetID,
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] UInt16[] dataArr,
			UInt16 usInElementCnt
			);



		// @13
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetIRCmdAfterLearn(
			Byte ucPort,
			Byte ucNetID,
			UInt16 usIrCmdNumber
			);


		// @14
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_ChkIrDataExistInBuffer(
			UInt16 usIrCmdNumber
			);



		// @15
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetIrDevCmdNameInBuffer(
			UInt16 usIrCmdNumber,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] Byte[] ucIrDevNameStr,
			UInt16 ucDevNameUcLen,
			ref UInt16 usDevNameByteArrOutLen,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 5)] Byte[] ucIrCmdNameStr,
			UInt16 ucIrCmdNameUcLen,
			ref UInt16 usIrCmdNameByteArrOutLen
			);




		// @16
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_SetIrDevCmdNameInBuffer(
			UInt16 usIrCmdNumber,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] Byte[] ucIrDevNameStr,
			UInt16 usDevNameUcLen,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 4)] Byte[] ucIrCmdNameStr,
			UInt16 usIrCmdNameUcLen
			);


		// @17
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_EmitIrSignal(
			Byte ucPort,
			Byte ucNetID,
			UInt16 usIrCmdNumber,
			Byte ucIrOutputCh
			);


		// @18
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_RunCommand(
			Byte ucPort,
			Byte ucNetID,
			UInt16 usIrCmdNumber,
			Byte ucIrOutputCh
			);


		// @19
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_DLOneIRDataToModule(
			Byte ucPort,
			Byte ucNetID,
			UInt16 usIrCmdNumber
			);



		// @20
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_LDOneIRDataToAPI(
			Byte ucPort,
			Byte ucNetID,
			UInt16 usIrCmdNumber
			);



		// @21
		[DllImport("irLrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_ResetIRModule(
			Byte ucPort
			);




		// @22
		[DllImport("irLrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetModbusRecvTimeout();




		// @23
		[DllImport("irLrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern void IR_SetModbusRecvTimeout(UInt16 recvTout);



		// @24
		[DllImport("irLrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_ClearOneIRLrnDataInAPI(
			UInt16 usIrCmdNumber,
			UInt16 usIrModuleIndex
			);



		// @25
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_ClearOneIRDeiveCmdNameInAPI(
			UInt16 usIrCmdNumber,
			UInt16 usIrModuleIndex
			);


		// @26
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_GetIRCmdSummaryBuffer(
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] UInt16[] dataArr,
			UInt16 usInElementCnt
			);


		// @27
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_SetIRCmdSummaryBuffer(
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] UInt16[] dataArr,
			UInt16 usInElementCnt
			);


		// @28
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_LoadIRCmdsFromFile(
			string filepath
			);

		// @29
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_SaveIRCmdsToFile(
			string filepath
			);


		// @47
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern void IR_ChkNIC(
			ref Int32 iNiCnt
			);



		// @48
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern void IR_GetNICInfo(
			Byte ucIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 2)] Byte[] szDescStr,
			Int32 usDescStrLen,
			ref Int32 ucRtnDecrLen,
			ref UInt32 ulIPAddr
			);




		// @49
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_EthSearchModules(
			UInt32 ulIPAddrNIC,
			ref Byte pucModuleCnt,
			UInt16 usTimeout
			);


		// @50
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern void IR_EthGetSearchModuleInfo(
			UInt16 usCntIndex,
			ref SearchRespEth ModuleInfo  /*out*/
			);



		// @60
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_TCPNew();

		// @61
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_TCPNew1(
			UInt32 ulNicIPAddr
			);

		// @62
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_TCPClose(Int32 hSocketTCP);


		//@63
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_Connect(
			Int32 hSocketTCP,
			UInt32 ulIPAddr,
			Byte ucNetID,
			UInt16 timeoutMs
			);


		
		// @69
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_GetModbusRecvTimeout();



		// @70
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern void IR_MTCP_SetModbusRecvTimeout(
			UInt16 recvTout
			);



		// @71
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_GetBasicSetting(
			Int32 hSocketTCP,
			ref BasicSetParamsMTCP pEBasicSettingMTCP  /*out*/
			);



		// @72
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_SetBasicSetting(
			Int32 hSocketTCP,
			[In]BasicSetParamsMTCP EBasicSettingMTCP, /*In*/
			Byte ucReboot
			);


		// @73
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_RebootIRModule(
			Int32 hSocketTCP
			);



		// @74
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_LrnModeSet(
			Int32 hSocketTCP,
			Byte ucMode
			);



		// @75
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_IsIRLearnOK(
			Int32 hSocketTCP,
			ref UInt16 pusIsOK,
			Int32 sTimeout
			);



		// @76
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_GetIRCmdAfterLearn(
			Int32 hSocketTCP,
			UInt16 usIrCmdNumber
			);




		// @77
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_EmitIrSignal(
			Int32 hSocketTCP,
			UInt16 usIrCmdNumber,
			Byte ucIrOutputCh
			);



		// @78
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_RunCommand(
			Int32 hSocketTCP,
			UInt16 usIrCmdNumber,
			Byte ucIrOutputCh
			);


		// @79
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_DLOneIRDataToModule(
			Int32 hSocketTCP,
			UInt16 usIrCmdNumber
			);


		// @81
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_LDOneIRDataToAPI(
			Int32 hSocketTCP,
			UInt16 usIrCmdNumber
			);



		// @82
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_ChkModuleName(
			Int32 hSocketTCP,
			ref UInt16 pwModuleIndex
			);


		// @83
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_GetIRCmdSummary(
			Int32 hSocketTCP,
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] UInt16[] dataArr,
			UInt16 usInElementCnt
			);





		// @84
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_SetIRCmdSummary(
			Int32 hSocketTCP,
			UInt16 usIrModuleIndex
			);



		// @85
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MTCP_SetIRCmdSummary1(
			Int32 hSocketTCP,
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] UInt16[] usDataArr,
			UInt16 usInElementCnt
			);













		// *************************************
		//  Modbus UDP Communication Functions
		// *************************************

		// @100
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_UDPNew(
			UInt32 ulIPAddr,
			Byte ucNetID
			);


		// @101
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_UDPNew1(
			UInt32 ulNicIPAddr,
			UInt32 ulIPAddr,
			Byte ucNetID
			);


		// @102
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_UDPClose(
			Int32 hSocketUDP
			);



		// @106
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_GetBasicSetting(
			Int32 hSocketUDP,
			ref BasicSetParamsMTCP pEBasicSettingMUDP
			);




		// @107
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_SetBasicSetting(
			Int32 hSocketUDP,
			BasicSetParamsMTCP EBasicSettingMUDP,
			Byte ucReset
			);




		// @108
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_RebootIRModule(
			Int32 hSocketUDP
			);


		// @109
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_GetModbusRecvTimeout();


		// @110
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_SetModbusRecvTimeout(
			UInt16 recvTout);



		// @111
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_LrnModeSet(
			Int32 hSocketUDP,
			Byte ucMode
			);


		// @112
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_IsIRLearnOK(
			Int32 hSocketUDP,
			ref UInt16 pusIsOK,
			Int32 sTimeout
			);


		// @113
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_EmitIrSignal(
			Int32 hSocketUDP,
			UInt16 usIrCmdNumber,
			Byte ucIrOutputCh
			);



		// @114
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_RunCommand(
			Int32 hSocketUDP,
			UInt16 usIrCmdNumber,
			Byte ucIrOutputCh
			);



		// @115
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_GetIRCmdAfterLearn(
			Int32 hSocketUDP,
			UInt16 usIrCmdNumber
			);


		// @116
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_DLOneIRDataToModule(
			Int32 hSocketUDP,
			UInt16 usIrCmdNumber
			);



		// @118
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_LDOneIRDataToAPI(
			Int32 hSocketUDP,
			UInt16 usIrCmdNumber
			);



		// @119
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_ChkModuleName(
			Int32 hSocketUDP,
			ref UInt16 pwModuleIndex
			);




		// @120
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_GetIRCmdSummary(
			Int32 hSocketUDP,
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] UInt16[] usDataArr,
			UInt16 usInElementCnt
			);





		// @121
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_SetIRCmdSummary(
			Int32 hSocketUDP,
			UInt16 usIrModuleIndex
			);




		// @122
		[DllImport("irlrnapi.dll", CallingConvention = CallingConvention.StdCall)]
		public static extern Int32 IR_MUDP_SetIRCmdSummary1(
			Int32 hSocketUDP,
			UInt16 usIrModuleIndex,
			[MarshalAs(UnmanagedType.LPArray, SizeParamIndex = 3)] UInt16[] usDataArr,
			UInt16 usInElementCnt
			);

	}
}








